"""GitHub issue template configuration management.

Manage GitHub issue templates in `.github/ISSUE_TEMPLATE/` directory:

- Bug report template (bug_report.yml)
- Feature request template (feature_request.yml)
- Template chooser config (config.yml)

See Also:
    https://docs.github.com/en/communities/using-templates-to-encourage-useful-issues-and-pull-requests
    pyrig.rig.configs.base.yml.YmlConfigFile
"""
