"""Python source file configuration management.

Manages Python source files for pyrig-based projects: package __init__.py files
(builders_init, configs_init, resources_init, src_init, tools_init),
CLI entry points (main.py), subcommands (subcommands.py, shared_subcommands.py),
and scratch files (.scratch.py).

See Also:
    pyrig.rig.configs.base.init.InitConfigFile
    pyrig.rig.configs.base.copy_module.CopyModuleConfigFile
"""
