"""Define custom configuration files.

Add custom `pyrig.rig.configs.base.base.ConfigFile` subclasses here to manage
project-specific configuration files. `ConfigFile` subclasses are automatically
discovered and initialized when running `pyrig mkroot`.
"""
